@@ -4,7 +4,7 @@
 
    Author: Michal Zalewski <lcamtuf@google.com>
 
-   Copyright 2009, 2010 by Google Inc. All Rights Reserved.
+   Copyright 2009, 2010, 2011 by Google Inc. All Rights Reserved.
 
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
@@ -68,7 +68,7 @@ static void resize_handler(int sig) {
 /* Usage info. */
 
 static void usage(char* argv0) {
-  SAY("Usage: %s [ options ... ] -o output_dir start_url [ start_url2 ... ]\n\n"
+  SAY("Usage: %s [ options ... ] -W wordlist -o output_dir start_url [ start_url2 ... ]\n\n"
 
       "Authentication and access options:\n\n"
 
@@ -77,6 +77,9 @@ static void usage(char* argv0) {
       "  -C name=val    - append a custom cookie to all requests\n"
       "  -H name=val    - append a custom HTTP header to all requests\n"
       "  -b (i|f|p)     - use headers consistent with MSIE / Firefox / iPhone\n"
+#ifdef PROXY_SUPPORT
+     "   -J proxy       - use a specified HTTP proxy server\n"
+#endif /* PROXY_SUPPORT */
       "  -N             - do not accept any new cookies\n\n"
 
       "Crawl scope options:\n\n"
@@ -107,9 +110,9 @@ static void usage(char* argv0) {
 
       "Dictionary management options:\n\n"
 
-      "  -W wordlist    - load an alternative wordlist (%s)\n"
+      "  -W wordlist    - use a specified read-write wordlist (required)\n"
+      "  -S wordlist    - load a supplemental read-only wordlist\n"
       "  -L             - do not auto-learn new keywords for the site\n"
-      "  -V             - do not update wordlist based on scan results\n"
       "  -Y             - do not fuzz extensions in directory brute-force\n"
       "  -R age         - purge words hit more than 'age' scans ago\n"
       "  -T name=val    - add new form auto-fill rule\n"
@@ -117,6 +120,7 @@ static void usage(char* argv0) {
 
       "Performance settings:\n\n"
 
+      "  -l max_req     - max requests per second (%f)\n"
       "  -g max_conn    - max simultaneous TCP connections, global (%u)\n"
       "  -m host_conn   - max simultaneous connections, per target IP (%u)\n"
       "  -f max_fail    - max number of consecutive HTTP errors (%u)\n"
@@ -126,10 +130,14 @@ static void usage(char* argv0) {
       "  -s s_limit     - response size limit (%u B)\n"
       "  -e             - do not keep binary responses for reporting\n\n"
 
+      "Safety settings:\n\n"
+
+      "  -k duration    - stop scanning after the given duration h:m:s\n\n"
+
       "Send comments and complaints to <lcamtuf@google.com>.\n", argv0,
-      max_depth, max_children, max_descendants, max_requests, DEF_WORDLIST,
-      MAX_GUESSES, max_connections, max_conn_host, max_fail, resp_tmout,
-      rw_tmout, idle_tmout, size_limit);
+      max_depth, max_children, max_descendants, max_requests,
+      MAX_GUESSES, max_requests_sec, max_connections, max_conn_host,
+      max_fail, resp_tmout, rw_tmout, idle_tmout, size_limit);
 
   exit(1);
 }
@@ -161,14 +169,14 @@ void splash_screen(void) {
 
       "More info: " cYEL "http://code.google.com/p/skipfish/wiki/KnownIssues\n\n" cBRI);
 
-  if (!no_fuzz_ext && (keyword_orig_cnt * extension_cnt) > 1000) {
+  if (!no_fuzz_ext && (keyword_orig_cnt * wg_extension_cnt) > 1000) {
 
     SAY(cLRD 
 
         "NOTE: The scanner is currently configured for directory brute-force attacks,\n"
         "and will make about " cBRI "%u" cLRD " requests per every fuzzable location. If this is\n"
         "not what you wanted, stop now and consult the documentation.\n\n",
-        keyword_orig_cnt * extension_cnt);
+        keyword_orig_cnt * wg_extension_cnt);
 
   }
 
@@ -181,13 +189,63 @@ void splash_screen(void) {
 #endif /* SHOW_SPLASH */
 
 
+/* Load URLs from file. */
+
+static void read_urls(u8* fn) {
+  FILE* f = fopen((char*)fn, "r");
+  u8    tmp[MAX_URL_LEN];
+  u32   loaded = 0;
+
+  if (!f) FATAL("Unable to open '%s'.", fn);
+
+  while (fgets((char*)tmp, MAX_URL_LEN, f)) {
+    struct http_request *req;
+    u8* url = tmp;
+    u32 l;
+
+    while (isspace(*url)) url++;
+
+    l = strlen((char*)url);
+    while (l && isspace(url[l-1])) l--;
+    url[l] = 0;
+
+    if (*url == '#' || !*url) continue;
+ 
+    req = ck_alloc(sizeof(struct http_request));
+
+    if (parse_url(url, req, NULL))
+      FATAL("Scan target '%s' in file '%s' is not a valid absolute URL.", url, fn);
+
+    if (!url_allowed_host(req))
+      APPEND_FILTER(allow_domains, num_allow_domains,
+                    __DFL_ck_strdup(req->host));
+
+    if (!url_allowed(req))
+      FATAL("URL '%s' in file '%s' explicitly excluded by -I / -X rules.",
+            url, fn);
+
+    maybe_add_pivot(req, NULL, 2);
+    destroy_request(req);
+    loaded++;
+
+  }
+
+  fclose(f);
+
+  if (!loaded) FATAL("No valid URLs found in '%s'.", fn);
+
+}
+
+
 /* Main entry point */
 
 int main(int argc, char** argv) {
   s32 opt;
   u32 loop_cnt = 0, purge_age = 0, seed;
-  u8 dont_save_words = 0, show_once = 0, be_quiet = 0, display_mode = 0;
-  u8 *wordlist = (u8*)DEF_WORDLIST, *output_dir = NULL;
+  u8  show_once = 0, be_quiet = 0, display_mode = 0, has_fake = 0;
+  u8 *wordlist = NULL, *output_dir = NULL;
+  u8* gtimeout_str = NULL;
+  u32 gtimeout = 0;
 
   struct termios term;
   struct timeval tv;
@@ -206,7 +264,8 @@ int main(int argc, char** argv) {
   SAY("skipfish version " VERSION " by <lcamtuf@google.com>\n");
 
   while ((opt = getopt(argc, argv,
-          "+A:F:C:H:b:Nd:c:x:r:p:I:X:D:POYQMZUEK:W:LVT:G:R:B:q:g:m:f:t:w:i:s:o:hue")) > 0)
+          "+A:B:C:D:EF:G:H:I:J:K:LMNOPQR:S:T:UW:X:YZ"
+          "b:c:d:ef:g:hi:k:l:m:o:p:q:r:s:t:uw:x:")) > 0)
 
     switch (opt) {
 
@@ -220,6 +279,18 @@ int main(int argc, char** argv) {
           break;
         }
 
+#ifdef PROXY_SUPPORT
+      case 'J':  {
+          u8* x = (u8*)strchr(optarg, ':');
+          if (!x) FATAL("Proxy data must be in 'host:port' form.");
+          *(x++) = 0;
+          use_proxy = (u8*)optarg;
+          use_proxy_port = atoi((char*)x);
+          if (!use_proxy_port) FATAL("Incorrect proxy port number.");
+          break;
+        }
+#endif /* PROXY_SUPPORT */
+
       case 'F': {
           u8* x = (u8*)strchr(optarg, '=');
           u32 fake_addr;
@@ -229,6 +300,7 @@ int main(int argc, char** argv) {
           if (fake_addr == (u32)-1)
             FATAL("Could not parse IP address '%s'.", x + 1);
           fake_host((u8*)optarg, fake_addr);
+          has_fake = 1;
           break;
         }
 
@@ -306,10 +378,6 @@ int main(int argc, char** argv) {
         no_parse = 1;
         break;
 
-      case 'V':
-        dont_save_words = 1;
-        break;
-
       case 'M':
         warn_mixed = 1;
         break;
@@ -356,7 +424,16 @@ int main(int argc, char** argv) {
         break;
 
       case 'W':
-        wordlist = (u8*)optarg;
+        if (wordlist)
+          FATAL("Only one -W parameter permitted (use -S to load supplemental dictionaries).");
+
+        if (!strcmp(optarg, "-")) wordlist = (u8*)"/dev/null";
+        else wordlist = (u8*)optarg;
+
+        break;
+
+      case 'S':
+        load_keywords((u8*)optarg, 1, 0);
         break;
 
       case 'b':
@@ -386,6 +463,11 @@ int main(int argc, char** argv) {
         if (!max_requests) FATAL("Invalid value '%s'.", optarg);
         break;
 
+      case 'l':
+        max_requests_sec = atof(optarg);
+        if (!max_requests_sec) FATAL("Invalid value '%s'.", optarg);
+        break;
+
       case 'f':
         max_fail = atoi(optarg);
         if (!max_fail) FATAL("Invalid value '%s'.", optarg);
@@ -430,6 +512,11 @@ int main(int argc, char** argv) {
         delete_bin = 1;
         break;
 
+      case 'k':
+        if (gtimeout_str) FATAL("Multiple -k options not allowed.");
+        gtimeout_str = (u8*)optarg;
+        break;
+
       case 'Z':
         no_500_dir = 1;
         break;
@@ -439,6 +526,11 @@ int main(int argc, char** argv) {
 
   }
 
+#ifdef PROXY_SUPPORT
+  if (has_fake && use_proxy)
+    FATAL("-F and -J should not be used together.");
+#endif /* PROXY_SUPPORT */
+
   if (access(ASSETS_DIR "/index.html", R_OK))
     PFATAL("Unable to access '%s/index.html' - wrong directory?", ASSETS_DIR);
 
@@ -456,16 +548,47 @@ int main(int argc, char** argv) {
   if (max_connections < max_conn_host)
     max_connections = max_conn_host;
 
-  load_keywords((u8*)wordlist, purge_age);
+  /* Parse the timeout string - format h:m:s */
+  if (gtimeout_str) {
+    int i = 0;
+    int m[3] = { 1, 60, 3600 };
 
-  /* Schedule all URLs in the command line for scanning */
+    u8* tok = (u8*)strtok((char*)gtimeout_str, ":");
+
+    while(tok && i <= 2) {
+      gtimeout += atoi((char*)tok) * m[i];
+      tok = (u8*)strtok(NULL, ":");
+      i++;
+    }
+
+    if(!gtimeout)
+      FATAL("Wrong timeout format, please use h:m:s (hours, minutes, seconds)");
+    DEBUG("* Scan timeout is set to %d seconds\n", gtimeout);
+  }
+
+
+  if (!wordlist)
+    FATAL("Wordlist not specified (try -h for help; see dictionaries/README-FIRST).");
+
+  load_keywords(wordlist, 0, purge_age);
+
+  /* Schedule all URLs in the command line for scanning. */
 
   while (optind < argc) {
 
-    struct http_request *req = ck_alloc(sizeof(struct http_request));
+    struct http_request *req;
+
+    /* Support @ notation for reading URL lists from files. */
+
+    if (argv[optind][0] == '@') {
+      read_urls((u8*)argv[optind++] + 1);
+      continue;
+    }
+
+    req = ck_alloc(sizeof(struct http_request));
 
     if (parse_url((u8*)argv[optind], req, NULL))
-      FATAL("One of specified scan targets is not a valid absolute URL.");
+      FATAL("Scan target '%s' is not a valid absolute URL.", argv[optind]);
 
     if (!url_allowed_host(req))
       APPEND_FILTER(allow_domains, num_allow_domains,
@@ -501,7 +624,23 @@ int main(int argc, char** argv) {
 
     u8 keybuf[8];
 
-    if (be_quiet || ((loop_cnt++ % 100) && !show_once)) continue;
+    u64 end_time;
+    u64 run_time;
+    struct timeval tv_tmp;
+
+    gettimeofday(&tv_tmp, NULL);
+    end_time = tv_tmp.tv_sec * 1000LL + tv_tmp.tv_usec / 1000;
+
+    run_time = end_time - st_time;
+    if (gtimeout > 0 && run_time && run_time/1000 > gtimeout) {
+      DEBUG("* Stopping scan due to timeout\n");
+      stop_soon = 1;
+    }
+
+    req_sec = (req_count - queue_cur / 1.15) * 1000 / (run_time + 1);
+
+    if (be_quiet || ((loop_cnt++ % 100) && !show_once && idle == 0))
+      continue;
 
     if (clear_screen) {
       SAY("\x1b[H\x1b[2J");
@@ -543,7 +682,7 @@ int main(int argc, char** argv) {
   tcsetattr(0, TCSANOW, &term);
   fcntl(0, F_SETFL, O_SYNC);
 
-  if (!dont_save_words) save_keywords((u8*)wordlist);
+  save_keywords((u8*)wordlist);
 
   write_report(output_dir, en_time - st_time, seed);
 
